/*
 * PgVecFld.java
 *
 * Created on January 7, 2005, 7:44 PM
 */

package GrapherODE;

import java.awt.Color;
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.object.PsPanel;
import jv.geom.PgPointSet;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.geom.PgVectorField;

/**
 *
 * @author David
 */

public class PgVecFld extends PgPointSet {
    protected       PuFunction                      m_func2;
    protected       PuInteger                       m_discr;
    protected       PgVectorField                   m_vf;
    protected       double[]                        m_domain;
    protected       boolean                         m_isSclVecs = false;
    protected       PuDouble                        m_veclength;
    protected       int                             indep, dep;
    protected       PdVector                        ICs;
    
    /** Creates a new instance of PgVectorField */
    public PgVecFld() {
        super(2);
       if (getClass() == PgVecFld.class) {
            init();
        }
    }
    
    public void init() {
        super.init();
        setName("Vector Field");
        
        m_vf = new PgVectorField(2, PgVectorField.VERTEX_BASED);
        
       m_discr = new PuInteger("Discr", this);
       m_discr.setDefBounds(2,300,1,5);
       m_discr.setDefValue(10);
       m_discr.init();
       
       m_veclength  = new PuDouble("Vector Length",this);
            m_veclength.setDefBounds(0,4,0.1,0.5);
            m_veclength.setDefValue(1);
            m_veclength.init();
        
        
        showVertices(false);
        m_vf.assureVectorColors();
        m_vf.showIndividualMaterial(true);
        m_vf.showVectorArrows(true);
        m_vf.setGlobalVectorColor(Color.black);
                    
 //     computeVf();
        
           
    }
    
    public void start() {
		if (PsDebug.NOTIFY) PsDebug.notify("start: ");
    }
    
    /** Draws Vector Field
     *  If there is only one equation (first order ODE), draws vector field for y vs t
     *  If there are more than one equation (2 var system), draws vector field for dep and indep
     *  
     *  @param domain   {xmin, ymin, xmax, ymax}
     *  @param func     PuFunction holding ODE
     *  @param  Indep   index of indep selected = m_2dx.  for 1st order = 1.  For sys, 0=x, 1=y.
     *  @param  Dep     index of dep selected = m_2dy[0]. for 1st order = 0.  For sys, 0=x, 1=y.  Vec Field 
     *                  is not drawn if indep = dep, so don't worry about that case.
     *  @param  InitCs     Pdvector of initial conditions.  Only used for systems.
     */
    public void drawVF(double[] domain, PuFunction func, int Indep, int Dep, PdVector InitCs) {
        indep = Indep;
        dep = Dep;
        ICs = InitCs;
        m_domain = domain;
        m_func2 = func;
        computeVf();
        m_vf.setGeometry(this);
        addVectorField(m_vf); 
    }
    
    public boolean update(Object event) {
		if (event == this) {
 //                       computeVf();
			return super.update(this);
		} 
                if (event == m_veclength) {
                    computeVf();
                }
                if (event == m_discr) {
                    computeVf();
                }
                super.update(this);
                return true;
    }
    
    public void showVF(boolean show) {
        showVectorFields(show);
    }
    
    public void computeVf() {
 
         //   int[] discr;
            double[] delta;
            delta = new double[2];
            int discr = m_discr.getValue();
            
            delta[0] = (m_domain[2] - m_domain[0])/(discr-1.);
            delta[1] = (m_domain[3] - m_domain[1])/(discr-1.);
             
            int numVertices = discr*discr;
            setNumVertices(numVertices);
            m_vf.setNumVectors(numVertices);
            
            int i,j;
            double u,v;
            
            int ind = 0;
  
                for (i=0; i<discr;i++){
                    for (j=0; j<discr;j++){
                        
                                u = m_domain[0]+delta[0]*i;  //value of indep
                                v = m_domain[1]+delta[1]*j;  //value of dep
                                
                                
				double x,y;
                                
                                if (m_func2.getNumFunctions() == 2) {
                                    
                                        if (dep == 0) { // x is dep, so y is indep
                                            double[] inp = {v,u,ICs.getEntry(2)};
                                            y = m_func2.eval(0,inp);
                                            x = m_func2.eval(1,inp);
                                        } else  { //y is dep, so x is indep
                                            double[] inp = {u,v,ICs.getEntry(2)};
                                            x = m_func2.eval(0,inp);
                                            y = m_func2.eval(1,inp);
                                        }
                                        
                                    
                                    
                                } else {
                                    double[] inp = {v,u};    // input is {y,t}
                                    x = 1.0;
                                    y = m_func2.eval(0,inp);
                                    
                                    
                                }
                               
                               
                                
                                double scl;
                                if (m_isSclVecs) {
                                    double length = Math.sqrt(x*x+y*y);
                                    scl = m_veclength.getValue()/(length+0.1);
                                } else {
                                    scl = m_veclength.getValue()/10.;
                                }
 //                               double length = Math.sqrt(x*x+y*y+z*z);
 //                               double scl = Math.min(deltaU,Math.min(deltaV,deltaW))/(length+0.1);
                      
                                setVertex(ind,u,v);
                                m_vf.setVector(ind,x*scl,y*scl);
                                ind++;
                        
                    }
                }
         
        }
    
}

